


CREATE OR REPLACE VIEW `view_groups` AS 
    SELECT  study_groups.ID         AS 'GroupID',
            study_groups.GroupNum   AS 'GroupNum',
            study_groups.Name       AS 'GroupName',
            grades.ID               AS 'GradeID',
            grades.Num              AS 'GradeNum',
            grades.Degree           AS 'Degree',
            specializations.ID      AS 'SpecID',
            specializations.Name    AS 'SpecName',
            specializations.Abbr    AS 'SpecAbbr',
            specializations.Code    AS 'SpecCode',
            faculties.ID        AS 'FacultyID', 
            faculties.Name      AS 'FacultyName', 
            faculties.Abbr      AS 'FacultyAbbr'
        FROM `study_groups`
        INNER JOIN `specializations`    ON study_groups.SpecializationID = specializations.ID
        INNER JOIN `grades` ON study_groups.GradeID = grades.ID
        INNER JOIN `faculties` ON faculties.ID = specializations.FacultyID; 


CREATE OR REPLACE VIEW `view_teachers` AS 
    SELECT  teachers.ID         AS 'TeacherID', 
            teachers.LastName, 
            teachers.FirstName, 
            teachers.SecondName, 
            teachers.AccountID,
            job_positions.ID    AS 'JobPositionID',
            job_positions.Name  AS 'JobPositionName', 
            departments.ID      AS 'DepID',
            departments.Name    AS 'DepName',
            faculties.ID        AS 'FacultyID', 
            faculties.Name      AS 'FacultyName', 
            faculties.Abbr      AS 'FacultyAbbr'
        FROM `teachers`
        INNER JOIN `departments`    ON  departments.ID = teachers.DepartmentID
        INNER JOIN `faculties`      ON  departments.FacultyID = faculties.ID
        INNER JOIN `job_positions`  ON  job_positions.ID = teachers.JobPositionID;


CREATE OR REPLACE VIEW `view_students` AS 
    SELECT  students.ID         AS 'StudentID',
            students.LastName,
            students.FirstName,
            students.SecondName,
            students.AccountID,
            students_groups.SemesterID,
            view_groups.*
        FROM `students`
        LEFT JOIN `students_groups` ON students_groups.StudentID = students.ID
        INNER JOIN `view_groups` ON view_groups.GroupID = students_groups.GroupID;


CREATE OR REPLACE VIEW `view_disciplines` AS 
    SELECT  disciplines.ID          AS 'DisciplineID',
            disciplines.AuthorID,
            disciplines.ExamType,
            disciplines.LectureCount,
            disciplines.PracticeCount,
            disciplines.LabCount,
            disciplines.SemesterID,
            disciplines.isLocked,
            disciplines.Milestone,
            disciplines.Subtype,
            grades.ID               AS 'GradeID',
            grades.Num              AS 'GradeNum',
            grades.Degree,
            subjects.ID             AS 'SubjectID',
            subjects.Name           AS 'SubjectName',
            subjects.Abbr           AS 'SubjectAbbr',
            faculties.ID            AS 'FacultyID',
            faculties.Name          AS 'FacultyName',
            faculties.Abbr          AS 'FacultyAbbr'
        FROM `disciplines`
        INNER JOIN `subjects` ON subjects.ID = disciplines.SubjectID
        INNER JOIN `faculties` ON faculties.ID = disciplines.FacultyID
        INNER JOIN `grades` ON grades.ID = disciplines.GradeID;


CREATE OR REPLACE VIEW `view_disciplines_results` AS
    SELECT  disciplines.ID AS 'DisciplineID',
            SUM(submodules.MaxRate) AS 'DisciplineRateMax',
            SUM(submodules.MaxRate*submodules.IsUsed) AS 'DisciplineRateCur'
        FROM `submodules`
        INNER JOIN `modules` ON modules.ID = submodules.ModuleID
        RIGHT JOIN `disciplines` ON disciplines.ID = modules.DisciplineID
        WHERE (modules.Type = 'regular') OR (modules.Type = 'exam' AND submodules.OrderNum = 1)
        GROUP BY disciplines.ID;


CREATE OR REPLACE VIEW `view_disciplines_teachers` AS 
    SELECT  view_disciplines.DisciplineID,
            view_teachers.*,
            (view_disciplines.AuthorID = view_teachers.TeacherID) AS 'IsAuthor'
        FROM `view_disciplines`
        LEFT JOIN `disciplines_teachers` ON disciplines_teachers.DisciplineID = view_disciplines.DisciplineID
        INNER JOIN `view_teachers` ON view_teachers.TeacherID = disciplines_teachers.TeacherID;


CREATE OR REPLACE VIEW `view_disciplines_students` AS 
    (SELECT disciplines_students.DisciplineID,
            disciplines_students.Type AS 'AttachType',
            view_students.*
        FROM `disciplines`
        LEFT JOIN `disciplines_students` ON disciplines_students.DisciplineID = disciplines.ID
        INNER JOIN `view_students` ON   view_students.StudentID = disciplines_students.StudentID AND
                                        view_students.SemesterID = disciplines.SemesterID
    ) UNION
    (SELECT disciplines_groups.DisciplineID,
            NULL AS 'AttachType',
            view_students.*
        FROM `disciplines`
        LEFT JOIN `disciplines_groups` ON disciplines_groups.DisciplineID = disciplines.ID
        LEFT JOIN `view_students` ON view_students.GroupID = disciplines_groups.GroupID AND
                                     view_students.SemesterID = disciplines.SemesterID
        WHERE NOT EXISTS 
            (SELECT disciplines_students.StudentID
                FROM `disciplines_students`
                WHERE disciplines_students.DisciplineID = disciplines_groups.DisciplineID AND
                    disciplines_students.StudentID = view_students.StudentID
            )
    );


CREATE OR REPLACE VIEW `view_roadmap` AS
    SELECT  modules.DisciplineID,
            modules.ID AS 'ModuleID',
            modules.Name AS 'ModuleName',
            modules.OrderNum AS 'ModuleOrderNum',
            modules.Type AS 'ModuleType', 
            # enum('regular','exam', 'bonus', 'extra')
            submodules.ID AS 'SubmoduleID',
            submodules.Name AS 'SubmoduleName',
            submodules.OrderNum AS 'SubmoduleOrderNum',
            submodules.MaxRate AS 'SubmoduleRate',
            submodules.Type AS 'SubmoduleType',
            # enum('CurrentControl','LandmarkControl')
            submodules.IsUsed AS 'SubmoduleIsUsed'
        FROM `modules`
        LEFT JOIN `submodules` ON submodules.ModuleID = modules.ID;


# without students, that haven't any rate
CREATE OR REPLACE VIEW `view_rating_result` AS
    SELECT  rating_table.StudentID,
            view_roadmap.DisciplineID,
            SUM(rating_table.Rate*(view_roadmap.ModuleType = 'regular')) AS 'RateRegular',
            SUM(rating_table.Rate*(view_roadmap.ModuleType = 'extra')) AS 'RateExtra',
            SUM(rating_table.Rate*(view_roadmap.ModuleType = 'bonus')) AS 'RateBonus',
            (SELECT rt.Rate*(vr.ModuleType = 'exam')
                FROM rating_table AS rt
                LEFT JOIN `view_roadmap` AS vr ON vr.SubmoduleID = rt.SubmoduleID
                WHERE vr.DisciplineID = view_roadmap.DisciplineID AND
                        rt.StudentID = rating_table.StudentID
                ORDER BY vr.ModuleType = 'exam' DESC,
                 (rt.Rate IS NULL) ASC,
                 vr.SubmoduleOrderNum DESC
                LIMIT 1
            ) AS 'RateExam'
        FROM `rating_table`
        LEFT JOIN `view_roadmap` ON view_roadmap.SubmoduleID = rating_table.SubmoduleID
        GROUP BY rating_table.StudentID, view_roadmap.DisciplineID;



